
%%                                                    RECOLOUR FIGURES 1.1

%                                                  Fabio Crameri, 23.05.21

%% NOTES:
% Only works with figures that don't apply shading.
clear, close all

%% Input
FigureFileName          	= 'CrameriEtAl2012_Fig1'; %Figure to recolour
FigureFormat                = '.jpg';
FigureFileDirectory     	= 'ExampleFigures/'; 

InteractiveColourmapSelector= logical(1);

newColourmap                = 'batlow';   %e.g., 'batlow' (sequential), 'roma' (diverging), bukavu (multi-sequential), romaO (cyclic)

CriticalRange           	= 0.05;     %Fiddle around with that value; ~0.05 +/-0.05. 0 means it is replacing only exact colour values.

OutputNumberColours         = 256;
flipNewColourmap            = logical(0);
removeBlackTicks            = logical(0);   %If colour map is obstructed by black ticks/lines, use this.
PickIntermediateValues      = logical(0);   %This helps to remove black ticks too.

SAVE.Plot                   = logical(1);
Figure_Name                 = FigureFileName;
SAVE.Directory              = FigureFileDirectory;
newColourmapString          = newColourmap;


if InteractiveColourmapSelector
    %% Read image file
    Im      = im2double(imread([FigureFileDirectory,FigureFileName,FigureFormat])); % read and convert to double

    %% Select colour bar
    figure(1),clf
    imshow(Im)
    hMsgBox	= msgbox({'Now outline the colourbar:';'';'Drag a rectangle over the colourmap without the colourbar edges. Double-click when finished.';'';'TIPP: First zoom into the figure to make a more accurate selction.'});

    roi     = images.roi.Rectangle('Color',[0.0,1.0,0.8],'StripeColor',[1.0,0.2,0.8],'LineWidth',0.75,'MarkerSize',4);
    draw(roi)
    wait(roi)     %Indicate completion by double-clicking the ROI object.
    
    Im2     = imcrop(Im,roi.Position);
    if logical(0)
        figure(2),clf
        imshow(Im2)
    end
    Im      = Im2;
    if exist('hMsgBox','var')
        delete(hMsgBox);
        clear('hMsgBox');
    end
else
    %% Read image file
    CMapFileName            = 'BeckerCmap.jpg'; %Cut out of the exact colourbar to recolour
    CMapFileDirectory       = '~/ETH/Programming/MatLabProgs/CreateColormaps/ColourMapImages/';
    Im = im2double(imread([CMapFileDirectory,CMapFileName])); % read and convert to double
end

%% Extract colour values
x   = size(Im,1);
y   = size(Im,2);

if x>y
    cmap(:,:) 	= Im(:,round(end/2),:);
else
    cmap(:,:) 	= Im(round(end/2),:,:);
end

%% Pick intermediate values
if PickIntermediateValues
    numPicks = 22;
    for iPick=1:numPicks+1
        dPick           = floor(size(cmap,1)/numPicks);
        if iPick==1
            dummy(iPick,:)  = cmap(1,:);
        else
            dummy(iPick,:)  = cmap((iPick-1)*dPick,:);
        end
    end
    cmap = dummy;
end

%% Clean up colour map
if removeBlackTicks
    for iCol=1:size(cmap,1)
        BlackTreshold = 0.1;
        if cmap(iCol,1)<=BlackTreshold && cmap(iCol,2)<=BlackTreshold && cmap(iCol,3)<=BlackTreshold
            if iCol==1
                cmap(iCol,:)   = cmap(iCol+1,:);
            else
                cmap(iCol,:)   = cmap(iCol-1,:);
            end
        end
    end
end

%% Create output colour map
for iRGB=1:3
%     cmapFinal(:,iRGB)   = interp1(1:OutputNumberColours,cmap(:,iRGB),linspace(1,OutputNumberColours,length(cmap)));
    cmapFinal(:,iRGB)   = interp1(1:length(cmap),cmap(:,iRGB),linspace(1,length(cmap),OutputNumberColours));

end

% if flipColourmap
%     cmapFinal   = flipud(cmapFinal);
% end

%% Test figure
figure(1),clf
contourf(peaks,100,'LineColor','none')
hc = colorbar; set(hc,'XTick',[],'YTick',[])
colormap(cmapFinal)
title('Extracted colour map','FontSize',16)
axis off

%% Replace pixel colours in original image with new ones
originalCmap    = cmapFinal;
numColours      = length(cmapFinal);

%% Read in existing colour map
if exist([newColourmap,'.mat'],'file')
    eval(['load(''',newColourmap,'.mat'');']);    %load('davos.mat');
end
eval(['newColourmap = ',newColourmap,';']);   	%newCmap = davos;
if flipNewColourmap
   newColourmap     = flipud(newColourmap);
end

%% Read original image file
Im  = im2double(imread([FigureFileDirectory,FigureFileName,FigureFormat])); % read and convert to double


%% Replace individual pixel colours
option1 = logical(1);
if option1
    R               = Im(:,:,1);
    G               = Im(:,:,2);
    B               = Im(:,:,3);
    PureWhite       = Im( R==1 & G==1 & B==1 ); %pure white
    PureBlack       = Im( R==0 & R==0 & R==0 ); %pure black
    
    Im2 = Im;
    R2 = R;
    G2 = G;
    B2 = B;
    hMsgBox = waitbar(0,'Processing image...');
    for iCol=1:numColours
        waitbar(iCol/numColours,hMsgBox,'Processing image...');
        
%         disp(num2str(iCol))
            
            R2(R>=originalCmap(iCol,1)-CriticalRange & R<=originalCmap(iCol,1)+CriticalRange & ... %matching colours
                G>=originalCmap(iCol,2)-CriticalRange & G<=originalCmap(iCol,2)+CriticalRange & ...
                B>=originalCmap(iCol,3)-CriticalRange & B<=originalCmap(iCol,3)+CriticalRange) = newColourmap(iCol,1);
            
            G2(R>=originalCmap(iCol,1)-CriticalRange & R<=originalCmap(iCol,1)+CriticalRange & ... %matching colours
                G>=originalCmap(iCol,2)-CriticalRange & G<=originalCmap(iCol,2)+CriticalRange & ...
                B>=originalCmap(iCol,3)-CriticalRange & B<=originalCmap(iCol,3)+CriticalRange) = newColourmap(iCol,2);
            
            B2(R>=originalCmap(iCol,1)-CriticalRange & R<=originalCmap(iCol,1)+CriticalRange & ... %matching colours
                G>=originalCmap(iCol,2)-CriticalRange & G<=originalCmap(iCol,2)+CriticalRange & ...
                B>=originalCmap(iCol,3)-CriticalRange & B<=originalCmap(iCol,3)+CriticalRange) = newColourmap(iCol,3);
            
    end
    close(hMsgBox)
    
    Im2(:,:,1) = R2;
    Im2(:,:,2) = G2;
    Im2(:,:,3) = B2;
    
    figure(1),clf
    imshow(Im2);

else
    RoundAccuracy               = 2;

    PixelColour = [NaN,NaN,NaN];
    for i=1:size(Im,1)
        for j=1:size(Im,2)
            disp(num2str(j))
            if (round(Im(i,j,1),RoundAccuracy)==1 && round(Im(i,j,2),RoundAccuracy)==1 && round(Im(i,j,3),RoundAccuracy)==1) || ... %pure white
                    (round(Im(i,j,1),RoundAccuracy)==0 && round(Im(i,j,2),RoundAccuracy)==0 && round(Im(i,j,3),RoundAccuracy)==0) %pure black
                continue
            end
            for iCol=1:numColours
                PixelColour(1:3) = Im(i,j,:);
                if PixelColour(1)>=originalCmap(iCol,1)-CriticalRange && PixelColour(1)<=originalCmap(iCol,1)+CriticalRange && ... %matching colours
                        PixelColour(2)>=originalCmap(iCol,2)-CriticalRange && PixelColour(2)<=originalCmap(iCol,2)+CriticalRange && ...
                        PixelColour(3)>=originalCmap(iCol,3)-CriticalRange && PixelColour(3)<=originalCmap(iCol,3)+CriticalRange
                    %replace colour
                    Im(i,j,:) = newColourmap(iCol,:);
                    
                    figure(1),clf
                    imshow(Im);
                    
                    disp(['Replacing ',num2str(round(Im(i,j,1),4),RoundAccuracy),' ',num2str(round(Im(i,j,2)),RoundAccuracy),' ',num2str(round(Im(i,j,3),RoundAccuracy)),' colour.'])
                    
                    continue
                end
            end
        end
    end
end

if SAVE.Plot
    Figure_Name = [Figure_Name,'_',newColourmapString];
    SWITCH.FullFigSize = logical(1);
    % saveSTAGYY( Figure_Nr, SaveDirectory, Figure_Name, save_jpg, save_eps, black_background, keep_aspect, jpg_resolution('-r300') )
    %     saveSTAGYY( 1, SaveDirectory, Figure_Name, save_jpg, save_eps, save_pdf, black_background, SWITCH.FullFigSize,'-r300')
    
    SAVE.FigureName             = Figure_Name;
    SAVE.FigureNr               = 1;
    SAVE.keepAspect             = logical(1);
    SAVE.jpgResolution       	= '-r300';
    SAVE.pngResolution       	= '-m4.5';
    f_saveFigure( SAVE );
end






